import os
import subprocess
import tempfile

from PyQt5.QtWidgets import QPushButton, QListWidget, QListWidgetItem, QApplication, QLabel, QWidget, QVBoxLayout
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt, QThread, pyqtSignal, QObject
from PyQt5 import uic
from Orange.widgets.widget import OWWidget, Output, Input
from Orange.data import Table, Domain, StringVariable
from Orange.widgets.settings import Setting


class FileDialogWorker(QObject):
    finished = pyqtSignal(list)

    def __init__(self, ps_filter):
        super().__init__()
        self.ps_filter = ps_filter

    def run(self):
        from PyQt5.QtWidgets import QApplication
        screen = QApplication.primaryScreen().geometry()
        x = screen.x() + (screen.width() // 2) - 400
        y = screen.y() + (screen.height() // 2) - 200

        ps_code = f"""
            Add-Type -AssemblyName System.Windows.Forms
            
            # Créer une nouvelle fenêtre parent invisible
            $parentForm = New-Object System.Windows.Forms.Form -Property @{{ 
                Size = New-Object System.Drawing.Size(0,0) # Taille minimale
                StartPosition = 'CenterScreen'
                TopMost = $true
                ShowInTaskbar = $false
                FormBorderStyle = 'None'
                Opacity = 0 # Rendre la fenêtre totalement transparente
            }}
            
            # Créer la boîte de dialogue de sélection de fichiers
            $openFileDialog = New-Object System.Windows.Forms.OpenFileDialog -Property @{{ 
                Filter = "{self.ps_filter}"
                Multiselect = $true
            }}
            
            # Afficher la boîte de dialogue en tant que fenêtre enfant de $parentForm
            $result = $openFileDialog.ShowDialog($parentForm)
            
            # Traiter les fichiers sélectionnés
            if ($result -eq [System.Windows.Forms.DialogResult]::OK) {{
                $selectedFiles = $openFileDialog.FileNames
                foreach ($file in $selectedFiles) {{
                    Write-Output $file
                }}
            }}
            
            # Fermer la fenêtre parent
            $parentForm.Close()
        """

        with tempfile.NamedTemporaryFile(delete=False, suffix=".ps1", mode="w", encoding="utf-8") as f:
            f.write(ps_code)
            ps_path = f.name

        result = subprocess.run(
            ["powershell", "-ExecutionPolicy", "Bypass", "-File", ps_path],
            capture_output=True, text=True
        )
        os.unlink(ps_path)

        file_paths = [line.strip() for line in result.stdout.strip().splitlines() if line.strip()]
        self.finished.emit(file_paths)


class OWFileExtSelector(OWWidget):
    name = "File Extension Selector"
    description = "Select multiple extensions and files"
    icon = "icons/file_extensor.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/file_extensor.png"
    category = "AAIT - Input"
    gui_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/ow_file_ext_selector.ui")
    priority = 10

    selected_extensions = Setting(["*.txt"])
    selected_paths = Setting([])

    class Inputs:
        in_data = Input("Data", Table)

    class Outputs:
        data = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.in_data = None
        self.overlay = None

        uic.loadUi(self.gui_path, self)

        self.extension_list = self.findChild(QListWidget, "listExtensions")
        self.file_button = self.findChild(QPushButton, "fileButton")

        self.extension_list.setSelectionMode(QListWidget.MultiSelection)
        self.populate_extensions()
        self.restore_selected_extensions()

        self.file_button.clicked.connect(self.select_files)
        self.extension_list.itemSelectionChanged.connect(self.update_settings)

    def loadSettings(self):
        super().loadSettings()

    def update_settings(self):
        self.selected_extensions = self.get_selected_extensions()
        self.saveSettings()

    def restore_selected_extensions(self):
        for i in range(self.extension_list.count()):
            item = self.extension_list.item(i)
            if item.flags() != Qt.NoItemFlags:
                ext = item.data(Qt.UserRole)
                if ext in self.selected_extensions:
                    item.setSelected(True)
        self.update_settings()

    def restore_selected_paths(self):
        if self.selected_paths:
            self.commit_paths()

    def populate_extensions(self):
        def add_category(label):
            item = QListWidgetItem(label)
            item.setFlags(Qt.NoItemFlags)
            font = QFont()
            font.setBold(True)
            item.setFont(font)
            item.setForeground(Qt.gray)
            self.extension_list.addItem(item)

        def add_extension(label, ext):
            item = QListWidgetItem(f"  {label}")
            item.setData(Qt.UserRole, ext)
            self.extension_list.addItem(item)

        extensions = {
            "Documents": [
                ("Text files (*.txt)", "*.txt"),
                ("Word documents (*.docx)", "*.docx"),
                ("PDF files (*.pdf)", "*.pdf"),
                ("Markdown (*.md)", "*.md"),
            ],
            "Tableurs": [
                ("Excel files (*.xlsx)", "*.xlsx"),
                ("CSV files (*.csv)", "*.csv"),
            ],
            "Images": [
                ("JPEG images (*.jpg *.jpeg)", "*.jpg *.jpeg"),
                ("PNG images (*.png)", "*.png"),
                ("GIF images (*.gif)", "*.gif"),
                ("SVG images (*.svg)", "*.svg"),
            ],
            "Audio": [
                ("MP3 (*.mp3)", "*.mp3"),
                ("WAV (*.wav)", "*.wav"),
            ],
            "Vidéo": [
                ("MP4 (*.mp4)", "*.mp4"),
                ("MKV (*.mkv)", "*.mkv"),
            ],
            "Archives": [
                ("ZIP (*.zip)", "*.zip"),
                ("RAR (*.rar)", "*.rar"),
                ("7-Zip (*.7z)", "*.7z"),
            ],
            "Code": [
                ("Python (*.py)", "*.py"),
                ("JSON (*.json)", "*.json"),
                ("HTML (*.html)", "*.html"),
                ("XML (*.xml)", "*.xml"),
                ("OWS (*.ows)", "*.ows"),
            ],
            "Autres": [
                ("All files (*.*)", "*.*")
            ]
        }

        for category, items in extensions.items():
            add_category(category)
            for label, ext in items:
                add_extension(label, ext)

    def get_selected_extensions(self):
        selected = [
            item.data(Qt.UserRole)
            for item in self.extension_list.selectedItems()
            if item.flags() != Qt.NoItemFlags
        ]
        return selected

    def show_overlay(self):
        if not self.overlay:
            self.overlay = QWidget()
            self.overlay.setWindowFlags(
                Qt.FramelessWindowHint |
                Qt.Tool  # ← PAS de WindowStaysOnTopHint
            )
            self.overlay.setAttribute(Qt.WA_TranslucentBackground)
            self.overlay.setAttribute(Qt.WA_TransparentForMouseEvents)
            self.overlay.setStyleSheet("background-color: rgba(0, 0, 0, 150);")

            layout = QVBoxLayout(self.overlay)
            message = QLabel("Veuillez sélectionner un fichier...")
            message.setStyleSheet("color: white; font-size: 24px;")
            message.setAlignment(Qt.AlignCenter)
            layout.addWidget(message)
            self.overlay.setLayout(layout)

            screen_geometry = QApplication.primaryScreen().geometry()
            self.overlay.setGeometry(screen_geometry)
            self.overlay.show()

            QApplication.processEvents()

    def hide_overlay(self):
        if self.overlay:
            self.overlay.hide()
            self.overlay = None

    def select_files(self):
        self.show_overlay()

        selected_exts = self.get_selected_extensions()
        if not selected_exts:
            self.warning("Please select at least one extension.")
            self.hide_overlay()
            return

        all_patterns = ";".join(selected_exts)
        ps_filter = f"Supported files ({all_patterns})|{all_patterns}"

        self.thread = QThread()
        self.worker = FileDialogWorker(ps_filter)
        self.worker.moveToThread(self.thread)

        self.thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.on_files_selected)
        self.worker.finished.connect(self.thread.quit)
        self.worker.finished.connect(self.worker.deleteLater)
        self.thread.finished.connect(self.thread.deleteLater)

        self.thread.start()

    def on_files_selected(self, file_paths):
        self.hide_overlay()
        if file_paths:
            self.selected_paths = file_paths
            self.commit_paths()

    def commit_paths(self):
        var = StringVariable("file_path")
        domain = Domain([], metas=[var])
        table = Table(domain, [[p] for p in self.selected_paths])
        self.Outputs.data.send(table)

    @Inputs.in_data
    def set_input_data(self, data):
        self.in_data = data

    def handleNewSignals(self):
        pass


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    ow = OWFileExtSelector()
    ow.show()
    sys.exit(app.exec_())



import os
import sys

import Orange.data
from AnyQt.QtWidgets import QApplication,QCheckBox

from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.IO4IT.ocr_function import word_converter
else:
    from orangecontrib.AAIT.utils import thread_management
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.IO4IT.ocr_function import word_converter

class OWwordpdf2docx(widget.OWWidget):
    name = "WordPdf2Docx"
    description = "Convert pdf from a directory to docx using word"
    icon = "icons/wordpdf2docx.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/wordpdf2docx.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/wordpdf2docx.ui")
    want_control_area = False
    priority = 3000

    class Inputs:
        data = Input("Data", Orange.data.Table)

    class Outputs:
        data = Output("Data", Orange.data.Table)

    @Inputs.data
    def set_data(self, in_data):
        self.data = in_data
        if self.autorun:
            self.run()


    def __init__(self):
        super().__init__()
        # Qt Management
        self.setFixedWidth(470)
        self.setFixedHeight(300)
        uic.loadUi(self.gui, self)
        self.check_box= self.findChild(QCheckBox, 'checkBox')
        # Data Management
        self.data = None
        self.thread = None
        self.autorun = True
        self.result = None
        self.post_initialized()

    def run(self):
        self.error("")
        # if thread is running quit
        if self.thread is not None:
            self.thread.safe_quit()

        if self.data is None:
            return


        # Verification of in_data
        self.error("")
        try:
            self.data.domain["input_dir"]
        except KeyError:
            self.error('You need a "input_dir" column in input data')
            return

        if type(self.data.domain["input_dir"]).__name__ != 'StringVariable':
            self.error('"input_dir" column needs to be a Text')
            return
        try:
            self.data.domain["output_dir"]
        except KeyError:
            self.error('You need a "output_dir" column in input data')
            return

        if type(self.data.domain["output_dir"]).__name__ != 'StringVariable':
            self.error('"output_dir" column needs to be a Text')
            return
        input_dir = self.data.get_column("input_dir")
        output_dir = self.data.get_column("output_dir")

        # Start progress bar
        self.progressBarInit()
        # Connect and start thread : main function, progress, result and finish
        # --> progress is used in the main function to track progress (with a callback)
        # --> result is used to collect the result from main function
        # --> finish is just an empty signal to indicate that the thread is finished
        ignore_existing_docx=False
        if self.check_box.isChecked():
            ignore_existing_docx=True
        self.thread = thread_management.Thread(word_converter.convert_pdf_structure, input_dir, output_dir,ignore_exsting_out_put=ignore_existing_docx)
        self.thread.progress.connect(self.handle_progress)
        self.thread.result.connect(self.handle_result)
        self.thread.finish.connect(self.handle_finish)
        self.thread.start()

    def handle_progress(self, value: float) -> None:
        self.progressBarSet(value)

    def handle_result(self, result):
        try:
            self.result = result
            self.error(result)
            self.Outputs.data.send(self.data)
        except Exception as e:
            print("An error occurred when sending out_data:", e)
            self.Outputs.data.send(None)
            return

    def handle_finish(self):
        print("conversion finished")
        self.progressBarFinished()

    def post_initialized(self):
        pass

if __name__ == "__main__":
    app = QApplication(sys.argv)
    my_widget = OWwordpdf2docx()
    my_widget.show()
    app.exec_()



import os
import boto3
from Orange.widgets.widget import OWWidget, Input
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from AnyQt.QtWidgets import QLineEdit, QFileDialog,QApplication
from Orange.data import Table

class OWS3FileDownloader(OWWidget):
    name = "S3 File Downloader"
    description = "Télécharge les fichiers listés depuis S3 vers un répertoire local."
    icon = "icons/download.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/download.png"
    priority = 20

    # Paramètres utilisateur
    access_key = Setting("")
    secret_key = Setting("")
    bucket_name = Setting("")
    download_path = Setting("")

    class Inputs:
        data = Input("Data", Table)

    def __init__(self):
        super().__init__()
        self.data = None

        # Interface utilisateur
        self.access_key_input = QLineEdit(self)
        self.access_key_input.setPlaceholderText("AWS Access Key")
        self.access_key_input.setText(self.access_key)
        self.access_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.access_key_input)

        self.secret_key_input = QLineEdit(self)
        self.secret_key_input.setPlaceholderText("AWS Secret Key")
        self.secret_key_input.setText(self.secret_key)
        self.secret_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.secret_key_input)

        self.bucket_input = QLineEdit(self)
        self.bucket_input.setPlaceholderText("Bucket S3")
        self.bucket_input.setText(self.bucket_name)
        self.bucket_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.bucket_input)

        self.path_button = gui.button(self.controlArea, self, "Choisir un répertoire", callback=self.select_directory)

        gui.button(self.controlArea, self, "Télécharger les fichiers", callback=self.download_files)

    def update_settings(self):
        self.access_key = self.access_key_input.text()
        self.secret_key = self.secret_key_input.text()
        self.bucket_name = self.bucket_input.text()

    def select_directory(self):
        directory = QFileDialog.getExistingDirectory(self, "Sélectionner un répertoire")
        if directory:
            self.download_path = directory

    @Inputs.data
    def set_data(self, data):
        self.data = data

    def download_files(self):
        if self.data is None:
            self.error("Aucune donnée reçue.")
            return

        if not self.download_path:
            self.error("Veuillez choisir un répertoire de téléchargement.")
            return

        try:
            session = boto3.Session(
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key
            )
            s3 = session.client("s3")
            for elem in self.data:
                file_key = elem["Nom"].value
                local_file_path = os.path.join(self.download_path, file_key)

                os.makedirs(os.path.dirname(local_file_path), exist_ok=True)
                s3.download_file(self.bucket_name, file_key, local_file_path)
            self.information("Téléchargement terminé !")

        except Exception as e:
            print(e)
            self.error(str(e))





import os
import sys
from Orange.data import Domain, StringVariable, Table
import Orange.data
from AnyQt.QtWidgets import QApplication
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output


if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from Orange.widgets.orangecontrib.AAIT.utils import thread_management
    from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
    from Orange.widgets.orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file
else:
    from orangecontrib.AAIT.utils import thread_management
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils.initialize_from_ini import apply_modification_from_python_file


@apply_modification_from_python_file(filepath_original_widget=__file__)
class OWMailLoader(widget.OWWidget):
    name = "OWMailLoader"
    description = "Load a mail from AAIT format"
    icon = ""
    #if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    #    icon = "icons_dev/owqueryllm.svg"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owmailloader.ui")
    want_control_area = True
    priority = 9999

    class Inputs:
        data = Input("Data", Orange.data.Table)

    class Outputs:
        data_out_all_dir = Output("Data Out All Dir", Orange.data.Table)
        data_out_nothing_to_do=Output("Data Out Nothing to Do", Orange.data.Table)

    @Inputs.data
    def set_data(self, in_data):
        self.valid_folders=[]
        input_dir_path=""
        self.error("")
        if in_data is None:
            return
        if not "input_dir" in in_data.domain:
            self.error("need input_dir in input data domain" )
            return
        if len(in_data)!=1:
            self.error("in data need to be exactly 1 line" )
            return
        input_dir_path=str(in_data[0]["input_dir"].value)
        input_dir_path.replace ("\\","/")

        self.valid_folders=self.get_valid_folders(input_dir_path)
        if len(self.valid_folders)==0:
            self.send_nothing_to_do()
            return
        print( self.valid_folders)
        self.run()


    def __init__(self):
        super().__init__()
        # Qt Management
        self.valid_folders=[]
        self.can_run = True
        self.setFixedWidth(700)
        self.setFixedHeight(500)
        uic.loadUi(self.gui, self)




        # # Data Management
        self.thread = None

        # Custom updates
        self.post_initialized()



    def get_valid_folders(self,input_dir_path):
        in_dir = os.path.join(input_dir_path, "in")

        # Étape 1 : Vérifier si le dossier existe
        if not os.path.isdir(in_dir):
            print(f"Dossier introuvable : {in_dir}")
            return []  # ou `return` selon ton besoin

        # Étape 2 : Parcourir les sous-dossiers
        valid_folders = []
        for name in os.listdir(in_dir):
            full_path = os.path.join(in_dir, name)
            if os.path.isdir(full_path):
                mail_ok_path = os.path.join(full_path, "mail.ok")
                if os.path.isfile(mail_ok_path):
                    valid_folders.append(full_path.replace("\\","/"))

        return valid_folders
    def send_nothing_to_do(self):

        # Définir une variable texte comme méta-attribut
        text_meta = StringVariable("nothing to do")

        # Créer un domaine sans variables principales, avec une méta
        domain = Domain([], metas=[text_meta])

        # Créer la table avec Table.from_list
        data_table = Table.from_list(domain, [["nothing"]])
        self.Outputs.data_out_nothing_to_do.send(data_table)

    def run(self):
        # Définir les variables de texte
        var_mail_txt = StringVariable("mail_path")
        var_pj_list = StringVariable("pj_files")

        # Définir le domaine avec deux colonnes texte
        domain = Domain([], metas=[var_mail_txt, var_pj_list])

        # Construire les lignes de données
        rows = []
        for folder in self.valid_folders:
            # Chemin vers mail.txt
            mail_path = os.path.join(folder, "mail.txt")

            # Chemin vers le sous-dossier "pj"
            pj_path = os.path.join(folder, "pj")

            # Lister les fichiers dans pj (ou chaîne vide s'il n'existe pas)
            if os.path.isdir(pj_path):
                files = os.listdir(pj_path)
            else:
                files = []

            # Convertir la liste de fichiers en chaîne (ex : "file1.pdf, file2.docx")
            files_str = ", ".join(files)
            files_str='['+files_str+']'
            # Ajouter la ligne
            rows.append([mail_path, files_str])

        # Créer la table Orange
        data_table = Table.from_list(domain, rows)

        # Afficher la table
        self.Outputs.data_out_all_dir.send(data_table)

    def post_initialized(self):
        pass

if __name__ == "__main__":
    app = QApplication(sys.argv)
    data = Orange.data.Table("C:/toto_ta_ta_titi/input.tab")
    my_widget = OWMailLoader()
    my_widget.show()
    my_widget.set_data(data)
    app.exec_()



import os
import logging
import urllib.parse
from pathlib import Path

from PyQt5.QtCore import QThread, pyqtSignal
from AnyQt.QtWidgets import QApplication, QLabel, QPushButton, QProgressBar, QListWidget

import Orange.data
from Orange.widgets import widget
from Orange.widgets.utils.signals import Input, Output
from Orange.data import Domain, StringVariable, Table

from docling.datamodel.base_models import InputFormat
from docling.datamodel.pipeline_options import PdfPipelineOptions
from docling.document_converter import DocumentConverter, PdfFormatOption
from docling_core.types.doc import ImageRefMode

_log = logging.getLogger(__name__)

IMAGE_RESOLUTION_SCALE = 2.0

class MarkdownConversionThread(QThread):
    result_signal = pyqtSignal(list)
    progress_signal = pyqtSignal(str, int)

    def __init__(self, input_dir, output_dir, parent=None):
        super().__init__(parent)
        self.input_dir = input_dir
        self.output_dir = output_dir

    def run(self):
        results = []
        files = list(self.input_dir.glob("*.pdf")) + list(self.input_dir.glob("*.docx"))
        total_files = len(files)
        processed = 0

        pipeline_options = PdfPipelineOptions()
        pipeline_options.images_scale = IMAGE_RESOLUTION_SCALE
        pipeline_options.generate_page_images = True
        pipeline_options.generate_picture_images = True

        doc_converter_pdf = DocumentConverter(
            format_options={InputFormat.PDF: PdfFormatOption(pipeline_options=pipeline_options)}
        )

        for idx, file_path in enumerate(files):
            try:
                output_subdir = self.output_dir / file_path.parent.relative_to(file_path.parents[1])
                output_subdir.mkdir(parents=True, exist_ok=True)

                doc_filename = file_path.stem + "_md-with-image-refs.md"
                output_file_path = output_subdir / doc_filename

                if output_file_path.exists():
                    print(f"🔁 Fichier déjà traité, ignoré : {doc_filename}")
                    results.append((doc_filename, output_file_path.read_text(encoding='utf-8')))
                    processed += 1
                    self.progress_signal.emit(doc_filename, int(processed / total_files * 100))
                    continue

                # Conversion
                if file_path.suffix.lower() == ".pdf":
                    conv_res = doc_converter_pdf.convert(file_path)
                else:
                    conv_res = DocumentConverter().convert(file_path)

                conv_res.document.save_as_markdown(output_file_path, image_mode=ImageRefMode.REFERENCED)

                with open(output_file_path, 'r', encoding='utf-8') as f:
                    content = urllib.parse.unquote(f.read())

                with open(output_file_path, 'w', encoding='utf-8') as f:
                    f.write(content)

                results.append((doc_filename, content))
                processed += 1
                self.progress_signal.emit(doc_filename, int(processed / total_files * 100))

            except Exception as e:
                print(f"❌ Erreur lors du traitement de {file_path}: {e}")
                continue

        self.result_signal.emit(results)


class FileProcessorApp(widget.OWWidget):
    name = "Markdownizer"
    description = "Convert PDFs, DOCX, TXT, CSV to Markdown and store in an output folder"
    icon = "icons/md.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/md.png"
    priority = 1001
    category = "Advanced Artificial Intelligence Tools"
    want_control_area = False

    class Inputs:
        data = Input("Input Directory", Orange.data.Table)

    class Outputs:
        data = Output("Markdown Data Table", Orange.data.Table)

    @Inputs.data
    def set_data(self, data):
        self.data = data
        if self.data is not None:
            path_index = None
            for i, meta_var in enumerate(self.data.domain.metas):
                if meta_var.name.lower() == 'input_dir':
                    path_index = i
                    break

            if path_index is not None:
                self.input_path = self.data.metas[0][path_index]
                print("Extracted input_dir:", self.input_path)
                self.startProcessing()
            else:
                print("No 'input_dir' column found in input data. Available columns:", [m.name for m in self.data.domain.metas])

    def __init__(self):
        super().__init__()
        self.initUI()
        self.data = None
        self.input_path = None

    def initUI(self):
        self.setGeometry(200, 200, 600, 400)
        self.mainArea.layout().setSpacing(10)
        self.status_label = QLabel("Sélectionnez un dossier contenant des fichiers.")
        self.progress_bar = QProgressBar()
        self.progress_bar.setValue(0)

        self.start_button = QPushButton("Démarrer le traitement")
        self.start_button.clicked.connect(self.startProcessing)

        self.file_list = QListWidget()

        self.mainArea.layout().addWidget(self.status_label)
        self.mainArea.layout().addWidget(self.progress_bar)
        self.mainArea.layout().addWidget(self.start_button)
        self.mainArea.layout().addWidget(self.file_list)

    def startProcessing(self):
        if not self.input_path:
            print("No valid input path found.")
            return

        input_dir = Path(self.input_path)
        if not input_dir.exists():
            print("Input directory does not exist:", input_dir)
            return

        self.output_dir = input_dir.parent / (input_dir.name + "_md")
        self.progress_bar.setValue(0)
        self.start_button.setEnabled(False)
        self.status_label.setText("Traitement en cours...")

        self.thread = MarkdownConversionThread(input_dir, self.output_dir)
        self.thread.result_signal.connect(self.handle_results)
        self.thread.progress_signal.connect(self.update_progress)
        self.thread.start()
        self.progressBarInit()

    def update_progress(self, filename, progress):
        print(f"File processed: {filename}")
        self.file_list.addItem(f"✅ {filename}")
        self.progress_bar.setValue(progress)

    def handle_results(self, results):
        self.processingComplete(results)
        self.progressBarFinished()

    def processingComplete(self, results):
        self.status_label.setText("Traitement terminé.")
        self.start_button.setEnabled(True)
        self.send_output(results)

    def send_output(self, results):
        domain = Domain([], metas=[
            StringVariable('input_dir'),
            StringVariable('output_dir'),
            StringVariable('name'),
            StringVariable('content')
        ])
        metas = [[
            str(self.input_path),
            str(self.output_dir),
            name,
            content
        ] for name, content in results] if results else [["", "", "", ""]]

        table = Table(domain, [[] for _ in metas])
        for i, meta in enumerate(metas):
            table.metas[i] = meta
        self.Outputs.data.send(table)

if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    window = FileProcessorApp()
    window.show()
    sys.exit(app.exec_())



import datetime
import ntpath
import os
import wave
import tempfile
import shutil
from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QApplication, QTextEdit, QPushButton, QSpinBox
from pyannote.audio import Audio
from pyannote.core import Segment

import numpy as np
import torch
import whisper
from Orange.data import Table, Domain, StringVariable
from Orange.widgets import widget
from Orange.widgets.utils.signals import Output
from sklearn.cluster import KMeans
from speechbrain.inference.speaker import EncoderClassifier

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils import SimpleDialogQt
    from orangecontrib.AAIT.utils.MetManagement import get_local_store_path, GetFromRemote
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils import SimpleDialogQt
    from orangecontrib.AAIT.utils.MetManagement import get_local_store_path, GetFromRemote

import subprocess

def convert_audio_to_pcm(file_path, ffmpeg_path):
    try:

        ext = os.path.splitext(file_path)[1].lower()
        if ext not in [".mp3", ".wav", ".m4a"]:
            print(f"[ERREUR] Type de fichier non supporté : {ext}")
            return None

        pcm_wav_path = file_path.replace(ext, "_pcm.wav")  # pas de nom compliqué ici

        ffmpeg_cmd = [
            ffmpeg_path, "-y",
            "-i", file_path,
            "-acodec", "pcm_s16le",
            "-ar", "16000",
            "-ac", "1",
            pcm_wav_path
        ]

        print(f"[INFO] Lancement de ffmpeg : {' '.join(ffmpeg_cmd)}")
        result = subprocess.run(ffmpeg_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

        if result.returncode != 0:
            print(f"[ERREUR] ffmpeg a échoué avec le code {result.returncode}")
            print("[STDERR]", result.stderr)
            return None

        if os.path.exists(pcm_wav_path):
            print(f"[INFO] Fichier converti : {pcm_wav_path}")
            return pcm_wav_path
        else:
            print("[ERREUR] Le fichier converti n’a pas été trouvé.")
            return None

    except Exception as e:
        print("❌ Exception pendant la conversion audio :", e)
        return None


def get_wav_duration(wav_path):
    try:
        with wave.open(wav_path, "rb") as wav_file:
            frames = wav_file.getnframes()
            rate = wav_file.getframerate()
            return frames / float(rate)
    except Exception as e:
        # and print the error message
        print("An error occurred when getting the  wav duration : ", e)
        return


class TranscriptionThread(QThread):
    result_signal = pyqtSignal(str, list, float)

    def __init__(self, file_path, model, embedding_model, audio_helper, num_speakers=2):
        super().__init__()
        self.file_path = file_path
        self.model = model
        self.embedding_model = embedding_model
        self.audio_helper = audio_helper
        self.num_speakers = num_speakers

    def run(self):
        try:
            print("[THREAD] Démarrage du thread de transcription")
            print(f"[THREAD] Fichier à traiter : {self.file_path}")

            if not os.path.exists(self.file_path):
                print(f"[ERREUR] Le fichier n'existe pas : {self.file_path}")
                self.result_signal.emit("Erreur : fichier introuvable.", [], 0.0)
                return

            file_duration = get_wav_duration(self.file_path)
            print(f"[INFO] Durée du fichier : {file_duration:.2f} sec" if file_duration else "[ERREUR] Durée inconnue")

            if file_duration is None:
                self.result_signal.emit("Error: File duration unknown.", [], 0.0)
                return

            print("[INFO] Début de la transcription avec Whisper")
            start_time = datetime.datetime.now()

            result = self.model.transcribe(
                self.file_path,
                language="fr",
                without_timestamps=False,
                temperature=0
            )

            transcription_time = (datetime.datetime.now() - start_time).total_seconds() / 60
            print(f"[INFO] Transcription terminée en {transcription_time:.2f} min")

            if not result or "segments" not in result or not result["segments"]:
                print("[ERREUR] Aucun segment détecté")
                self.result_signal.emit("Error: No speech detected.", [], transcription_time)
                return

            segments = result["segments"]
            print(f"[INFO] Nombre de segments détectés : {len(segments)}")

            embeddings = None

            for i, segment in enumerate(segments):
                start, end = segment["start"], segment["end"]

                if end > file_duration:
                    print(f"[AVERTISSEMENT] Segment {i} ignoré (fin hors durée)")
                    continue

                try:
                    waveform, _ = self.audio_helper.crop(self.file_path, Segment(start, end))
                    if waveform.ndim == 1:
                        waveform = waveform.unsqueeze(0)
                    with torch.no_grad():
                        embedding = self.embedding_model.encode_batch(waveform).squeeze().cpu().numpy()
                    if embeddings is None:
                        embeddings = np.zeros((len(segments), embedding.shape[0]))
                    embeddings[i] = embedding
                except Exception as crop_err:
                    print(f"[ERREUR] Erreur lors du crop ou de l'embedding pour le segment {i} : {crop_err}")
                    continue

            print("[INFO] Clustering des embeddings avec KMeans")
            clustering = KMeans(n_clusters=min(self.num_speakers, len(segments)), random_state=42).fit(embeddings)
            labels = clustering.labels_

            speaker_map = {}
            merged_segments = []
            current_speaker = None
            current_text = ""
            current_start = None
            table_output = []

            print("[INFO] Regroupement par locuteur")
            for i, segment in enumerate(segments):
                speaker_id = labels[i]
                if speaker_id not in speaker_map:
                    speaker_map[speaker_id] = f"SPEAKER {len(speaker_map) + 1}"
                speaker = speaker_map[speaker_id]

                if current_speaker == speaker:
                    current_text += f" {segment['text']}"
                else:
                    if current_speaker is not None:
                        timestamp = str(datetime.timedelta(seconds=round(current_start)))
                        merged_segments.append(f"{current_speaker} {timestamp}: {current_text}")
                        table_output.append([current_speaker, timestamp, current_text])
                    current_speaker = speaker
                    current_text = segment["text"]
                    current_start = segment["start"]

            print("[INFO] Finalisation des résultats")
            speaker_text_output = "\n".join(merged_segments)
            self.result_signal.emit(speaker_text_output, table_output, transcription_time)
            print("[THREAD] Transcription terminée et signal émis")

        except Exception as e:
            print("❌ An error occurred during transcription:", e)
            import traceback
            traceback.print_exc()
            return


class OWSpeech_To_Text(widget.OWWidget):
    name = "Speech To Text"
    description = "Convert audio to text with speaker recognition"
    priority = 1111
    category = "Advanced Artificial Intelligence Tools"
    icon = "icons/speech_to_text.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/speech_to_text.png"
    gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owspeechtotext.ui")

    class Outputs:
        data = Output("Data", Table)
        global_transcription = Output("Global Transcription", Table)

    def __init__(self):
        super().__init__()
        self.file_path = ""
        self.num_speakers = 4 # spin box a defaut 4

        self.local_store_path = get_local_store_path()
        model_name = "small.pt"
        self.embedding_model_name = "spkrec-ecapa-voxceleb"
        self.model_path = os.path.join(self.local_store_path, "Models", "S2T", model_name)
        self.embedding_model_path = os.path.join(self.local_store_path, "Models", "S2T", self.embedding_model_name)

        self.ffmpeg_path = os.path.join(self.local_store_path, "Models", "S2T", "ffmpeg", "bin", "ffmpeg.exe")
        # Extraire le dossier de ffmpeg.exe
        ffmpeg_bin_dir = os.path.dirname(self.ffmpeg_path)

        # Ajouter ffmpeg au PATH pour whisper/ffmpeg-python
        if ffmpeg_bin_dir not in os.environ["PATH"]:
            os.environ["PATH"] = ffmpeg_bin_dir + os.pathsep + os.environ["PATH"]
            print(f"[INFO] Dossier ffmpeg ajouté au PATH Python : {ffmpeg_bin_dir}")


        if not os.path.exists(self.model_path):
            if not SimpleDialogQt.BoxYesNo("Whisper turbo Transcription Model isn't in your computer. Do you want to download it from AAIT store?"):
                return
            try:
                if 0 != GetFromRemote("Whisper turbo"):
                    return
            except Exception as e:
                print(e)
                SimpleDialogQt.BoxError("Unable to get the Whisper turbo.")
                return

        if not os.path.exists(self.embedding_model_path):
            if not SimpleDialogQt.BoxYesNo("Voxceleb Embedding Model isn't in your computer. Do you want to download it from AAIT store?"):
                return
            try:
                if 0 != GetFromRemote("Voxceleb"):
                    return
            except Exception as e:
                print(e)
                SimpleDialogQt.BoxError("Unable to get the Voxceleb.")
                return
        if not os.path.exists(self.ffmpeg_path):
            if not SimpleDialogQt.BoxYesNo("FFMPEG isn't in your computer. Do you want to download it from AAIT store?"):
                return
            try:
                if 0 != GetFromRemote("FFMPEG"):
                    return
            except Exception as e:
                print(e)
                SimpleDialogQt.BoxError("Unable to get the ffmpeg.")
                return

        self.model = whisper.load_model(self.model_path)
        print("Version of PyTorch :", torch.__version__)
        print("Used cuda version :", torch.version.cuda)
        print("CUDA available ? :", torch.cuda.is_available())
        if torch.cuda.is_available():
            print("Number of GPU :", torch.cuda.device_count())
            print("GPU's name :", torch.cuda.get_device_name(0))
        # Définition d'un model d'embedding (hard codé pour l'instant)

        self.embedding_model = EncoderClassifier.from_hparams(source=self.embedding_model_path)
        self.audio_helper = Audio()
        uic.loadUi(self.gui, self)

        self.file_button = self.findChild(QPushButton,
                                          'fileButton')
        self.file_button.clicked.connect(self.select_file)
        self.process_button = self.findChild(QPushButton,
                                             'processButton')
        self.process_button.clicked.connect(self.process_recording)
        self.text_area = self.findChild(QTextEdit, 'textArea')

        self.spinBox_nb_people=self.findChild(QSpinBox,'spinBox_nb_people')
        self.spinBox_nb_people.setValue(int(self.num_speakers))
        self.spinBox_nb_people.valueChanged.connect(self.spinbox_value_changed)
        self.process_button.setEnabled(False)

    def spinbox_value_changed(self, value):
        self.num_speakers = value

    def select_file(self):
        file_path = SimpleDialogQt.BoxSelectExistingFile(self, extention="Audio files (*.wav *.mp3 *.m4a)")

        if file_path:
            # 🔃 Copie dans un chemin sans accents ni caractères spéciaux
            temp_dir = tempfile.gettempdir()
            base_ext = os.path.splitext(file_path)[1]
            clean_copy = os.path.join(temp_dir, "input_audio" + base_ext)
            shutil.copy(file_path, clean_copy)
            print(f"[INFO] Copie vers fichier temporaire sans accents : {clean_copy}")

            # 🔁 Conversion dans ce dossier temporaire
            pcm_path = convert_audio_to_pcm(clean_copy, self.ffmpeg_path)
            print(f"[DEBUG] pcm_path: {pcm_path}")

            if pcm_path:
                self.file_path = pcm_path
                self.temp_pcm_path = pcm_path
                self.process_button.setEnabled(True)
            else:
                SimpleDialogQt.BoxError("Erreur : La conversion audio a échoué.")
        else:
            print("[ERREUR] Aucun fichier sélectionné.")


    def process_recording(self):
        self.process_button.setEnabled(False)
        if not self.file_path:
            SimpleDialogQt.BoxError(
                "Aucun fichier sélectionné. Veuillez choisir un fichier audio avant de lancer la transcription.")
            return
        self.num_speakers = self.spinBox_nb_people.value()
        self.text_area.setText("Transcription in progress...")
        self.thread = TranscriptionThread(
            self.file_path, self.model, self.embedding_model, self.audio_helper, self.num_speakers
        )
        self.thread.result_signal.connect(self.display_text)
        self.thread.start()
        self.progressBarInit()  # Ajout de la barre de progression

    def display_text(self, text, table_output, transcription_time):
        self.text_area.setText(f"{text}\n\n⏳ Temps de transcription: {transcription_time:.2f} minutes")

        # Sortie 1 : tableau par speaker
        domain = Domain([],
                        metas=[StringVariable("Speaker"), StringVariable("Timestamp"), StringVariable("Transcription")])
        metas = [[row[0], row[1], row[2]] for row in table_output] if table_output else [["", "", ""]]
        out_data = Table(domain, [[] for _ in metas])
        for i, meta in enumerate(metas):
            out_data.metas[i] = meta
        self.Outputs.data.send(out_data)

        # Sortie 2 : une seule ligne avec toutes les infos
        global_domain = Domain([],
                               metas=[StringVariable("Nom du fichier"),
                                      StringVariable("Transcription"),
                                      StringVariable("Temps de transcription (min)")])
        filename = ntpath.basename(self.file_path)  # Pour ne garder que le nom du fichier
        global_metas = [[filename, text, f"{transcription_time:.2f}"]]
        global_table = Table(global_domain, [[]])
        global_table.metas[0] = global_metas[0]
        self.Outputs.global_transcription.send(global_table)
        self.progressBarFinished()

if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    window = OWSpeech_To_Text()
    window.show()
    sys.exit(app.exec_())


import boto3
from Orange.widgets.widget import OWWidget, Input, Output
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from AnyQt.QtWidgets import QLineEdit, QApplication
from Orange.data import Table, Domain, StringVariable, ContinuousVariable
import os

class OWS3FileLister(OWWidget):
    name = "S3 File Lister"
    description = "Liste les fichiers d'un bucket S3 et affiche leurs détails."
    icon = "icons/list_aws.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/list_aws.png"
    priority = 10

    # Paramètres utilisateur
    access_key = Setting("")
    secret_key = Setting("")
    bucket_name = Setting("")
    allowed_extensions = Setting("*")
    folder_name = Setting("")

    class Outputs:
        data = Output("Data", Table)

    def __init__(self):
        super().__init__()

        # Interface utilisateur
        self.access_key_input = QLineEdit(self)
        self.access_key_input.setPlaceholderText("AWS Access Key")
        self.access_key_input.setText(self.access_key)
        self.access_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.access_key_input)

        self.secret_key_input = QLineEdit(self)
        self.secret_key_input.setPlaceholderText("AWS Secret Key")
        self.secret_key_input.setText(self.secret_key)
        self.secret_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.secret_key_input)

        self.bucket_input = QLineEdit(self)
        self.bucket_input.setPlaceholderText("Bucket S3")
        self.bucket_input.setText(self.bucket_name)
        self.bucket_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.bucket_input)

        self.extensions_input = QLineEdit(self)
        self.extensions_input.setPlaceholderText("Extensions files from S3")
        self.extensions_input.setText(",".join(self.allowed_extensions))
        self.extensions_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.extensions_input)

        # self.folder_input = QLineEdit(self)
        # self.folder_input.setPlaceholderText("Folder files from S3")
        # self.folder_input.setText(self.folder_name)
        # self.folder_input.editingFinished.connect(self.update_settings)
        # gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.extensions_input)

        gui.button(self.controlArea, self, "Lister les fichiers", callback=self.list_files)

    def update_settings(self):
        self.access_key = self.access_key_input.text()
        self.secret_key = self.secret_key_input.text()
        self.bucket_name = self.bucket_input.text()
        self.allowed_extensions = self.extensions_input.text()
        self.folder_name = self.folder_input.text()

    def list_files(self):
        self.error(None)
        try:
            session = boto3.Session(
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key
            )
            s3 = session.client("s3")
            response = s3.list_objects_v2(Bucket=self.bucket_name, Prefix = self.folder_name)
            if self.allowed_extensions == "*":
                self.allowed_extensions = "*"
            elif isinstance(self.allowed_extensions, str):
                self.allowed_extensions = tuple(ext.strip().lower() for ext in self.allowed_extensions.split(","))

            if "Contents" in response:
                data = []
                meta = []
                for obj in response["Contents"]:
                    if self.allowed_extensions == "*" or self.allowed_extensions == "" or obj["Key"].lower().endswith(self.allowed_extensions):
                        data.append([str(obj["Size"])])
                        meta.append([obj["Key"],obj["LastModified"].strftime("%Y-%m-%d %H:%M:%S")])

            domain = Domain(
                [ContinuousVariable("Taille (bytes)")], metas=[StringVariable("Nom"), StringVariable("Date")]
            )
            orange_table = Table.from_numpy(domain, data, metas=meta)
            self.Outputs.data.send(orange_table)

        except Exception as e:
            self.error(str(e))

if __name__ == "__main__":
    import sys

    app = QApplication(sys.argv)
    window = OWS3FileLister()
    window.show()
    sys.exit(app.exec_())



import time
import warnings
from setuptools import setup, find_packages
import sys
import subprocess
import platform
from setuptools.command.install import install

def log_message(message, level="INFO"):
    timestamp = time.strftime("%H:%M:%S")
    border = "=" * (len(message) + 4)
    print(f"\n{border}")
    print(f"{timestamp} - {level} - {message}")
    print(f"{border}\n")

class CustomInstall(install):
    def run(self):
        log_message(f"Début de l'installation de {NAME} v{VERSION}")
        log_message("Étape 1/2 : Installation des dépendances principales")
        install.run(self)

        if 'cuda' in sys.argv:
            log_message("Option [cuda] détectée", "IMPORTANT")
            log_message("⚠️ Assurez-vous d’utiliser --extra-index-url https://download.pytorch.org/whl/cu126", "NOTE")
        else:
            log_message("Mode CPU sélectionné", "NOTE")
            self._warn_cuda_option()

    def _warn_cuda_option(self):
        if platform.system() == "Windows":
            log_message("ASTUCE: Pour activer CUDA sous Windows", "NOTE")
            print("\033[94m" + "="*80)
            print("Pour une version avec accélération GPU, utilisez :")
            print("pip install io4it[cuda] --extra-index-url https://download.pytorch.org/whl/cu126")
            print("="*80 + "\033[0m\n")

# Configuration
NAME = "io4it"
VERSION = "1.0.0.1"

INSTALL_REQUIRES = [
    "boto3",
    "docling",
    "docling-core", 
    "speechbrain",
    "whisper",
    "whisper-openai",
    "pyannote.audio",
    "pyannote-core",
    "wave",
    "scikit-learn",
]

AUTHOR = ""
AUTHOR_EMAIL = ""
URL = ""
DESCRIPTION = ""
LICENSE = ""
KEYWORDS = ["orange3 add-on",]

PACKAGES = find_packages()
PACKAGES = [pack for pack in PACKAGES if "orangecontrib" in pack and "IO4IT" in pack]
PACKAGES.append("orangecontrib")

PACKAGE_DATA = {
    "orangecontrib.IO4IT.widgets": ["icons/*", "designer/*"],
}


ENTRY_POINTS = {
    "orange.widgets": (
        "Advanced Artificial Intelligence Tools = orangecontrib.IO4IT.widgets",
    )
}

NAMESPACE_PACKAGES = ["orangecontrib"]


setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license=LICENSE,
    keywords=KEYWORDS,
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    install_requires=INSTALL_REQUIRES,
    entry_points=ENTRY_POINTS,
    namespace_packages=NAMESPACE_PACKAGES,
)



import os
from PyQt5.QtWidgets import QPushButton, QListWidget, QListWidgetItem, QApplication, QWidget, QVBoxLayout, QSizePolicy
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt
from PyQt5 import uic
from AnyQt.QtWidgets import QFileDialog
from Orange.widgets.widget import OWWidget, Output, Input
from Orange.data import Table, Domain, StringVariable
from Orange.widgets.settings import Setting

if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils import SimpleDialogQt
    from orangecontrib.AAIT.utils.MetManagement import get_local_store_path, GetFromRemote
else:
    from orangecontrib.AAIT.utils.import_uic import uic
    from orangecontrib.AAIT.utils import SimpleDialogQt
    from orangecontrib.AAIT.utils.MetManagement import get_local_store_path, GetFromRemote

from PyQt5.QtWebEngineWidgets import QWebEngineView
from PyQt5.QtCore import QUrl


class OWFileExtSelector(OWWidget):
    name = "Javascript visualization"
    description = "Select multiple extensions and files"
    icon = "icons/visualizationer.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/visualizationer.png"
    category = "AAIT - Input"
    gui_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/owvisualizationer.ui")
    priority = 10


    class Inputs:
        in_data = Input("Data", Table)

    class Outputs:
        data = Output("Data", Table)

    def __init__(self):
        super().__init__()
        self.in_data = None

        # Load UI
        uic.loadUi(self.gui_path, self)

        self.web_frame = self.findChild(QWidget, "webFrame")  # Doit exister dans le .ui
        self.web_view = QWebEngineView(self.web_frame)
        self.web_view.setMinimumSize(400, 300)
        self.web_view.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # Crée un layout si nécessaire
        layout = QVBoxLayout(self.web_frame)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.web_view)

        # Charge ton fichier HTML local
        html_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "designer/chart.html"))
        self.web_view.load(QUrl.fromLocalFile(html_path))


    @Inputs.in_data
    def set_input_data(self, data):
        self.in_data = data


if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    ow = OWFileExtSelector()
    ow.show()
    sys.exit(app.exec_())



import os
import boto3
from Orange.widgets.widget import OWWidget, Input
from Orange.widgets.settings import Setting
from Orange.widgets import gui
from AnyQt.QtWidgets import QLineEdit, QFileDialog,QApplication
from Orange.data import Table


class OWS3FileDownloader(OWWidget):
    name = "S3 File Uploader"
    description = "Envoies les fichiers listés depuis un répertoire local vers s3."
    icon = "icons/upload.png"
    if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
        icon = "icons_dev/upload.png"
    priority = 20

    # Paramètres utilisateur
    access_key = Setting("")
    secret_key = Setting("")
    bucket_name = Setting("")
    download_path = Setting("")

    class Inputs:
        data = Input("Data", Table)

    def __init__(self):
        super().__init__()
        self.data = None
        # Interface utilisateur
        self.access_key_input = QLineEdit(self)
        self.access_key_input.setPlaceholderText("AWS Access Key")
        self.access_key_input.setText(self.access_key)
        self.access_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.access_key_input)

        self.secret_key_input = QLineEdit(self)
        self.secret_key_input.setPlaceholderText("AWS Secret Key")
        self.secret_key_input.setText(self.secret_key)
        self.secret_key_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.secret_key_input)

        self.bucket_input = QLineEdit(self)
        self.bucket_input.setPlaceholderText("Bucket S3")
        self.bucket_input.setText(self.bucket_name)
        self.bucket_input.editingFinished.connect(self.update_settings)
        gui.widgetBox(self.controlArea, orientation='vertical').layout().addWidget(self.bucket_input)

        self.path_button = gui.button(self.controlArea, self, "Choisir un répertoire", callback=self.select_directory)

        gui.button(self.controlArea, self, "Envoyer les fichiers dans s3", callback=self.upload_files)

    def update_settings(self):
        self.access_key = self.access_key_input.text()
        self.secret_key = self.secret_key_input.text()
        self.bucket_name = self.bucket_input.text()

    def select_directory(self):
        directory = QFileDialog.getExistingDirectory(self, "Sélectionner un répertoire")
        if directory:
            self.download_path = directory

    @Inputs.data
    def set_data(self, data):
        self.data = data

    def upload_files(self):
        if self.data is None:
            self.error("Aucune donnée reçue.")
            return

        if not self.download_path:
            self.error("Veuillez choisir un répertoire de téléchargement.")
            return

        try:
            session = boto3.Session(
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key
            )
            s3 = session.client("s3")
            files = os.listdir(self.download_path)  # Liste tout (fichiers + dossiers)
            files_only = [f for f in files if os.path.isfile(os.path.join(self.download_path, f))]

            for file in files_only:
                s3.upload_file(self.download_path + "/" + file, "ifia", file)
            self.information("Téléchargement terminé !")
        except Exception as e:
            print(e)
            self.error(str(e))


